from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app import crud, models, schemas
from app.api import deps

router = APIRouter()


@router.get("/", response_model=List[schemas.Service])
def read_services(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
) -> Any:
    """
    Retrieve services.
    """
    services = crud.service.get_active(db, skip=skip, limit=limit)
    return services


@router.post("/", response_model=schemas.Service)
def create_service(
    *,
    db: Session = Depends(deps.get_db),
    service_in: schemas.ServiceCreate,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Create new service.
    """
    service = crud.service.get_by_slug(db, slug=service_in.slug)
    if service:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="A service with this slug already exists",
        )
    service = crud.service.create(db, obj_in=service_in)
    return service


@router.get("/{slug}", response_model=schemas.Service)
def read_service(
    *,
    db: Session = Depends(deps.get_db),
    slug: str,
) -> Any:
    """
    Get service by slug.
    """
    service = crud.service.get_by_slug(db, slug=slug)
    if not service:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Service not found",
        )
    return service


@router.put("/{id}", response_model=schemas.Service)
def update_service(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    service_in: schemas.ServiceUpdate,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Update a service.
    """
    service = crud.service.get(db, id=id)
    if not service:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Service not found",
        )
    service = crud.service.update(db, db_obj=service, obj_in=service_in)
    return service


@router.delete("/{id}", response_model=schemas.Service)
def delete_service(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Delete a service.
    """
    service = crud.service.get(db, id=id)
    if not service:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Service not found",
        )
    service = crud.service.remove(db, id=id)
    return service