from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app import crud, models, schemas
from app.api import deps

router = APIRouter()


@router.get("/", response_model=List[schemas.TeamMember])
def read_team_members(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
) -> Any:
    """
    Retrieve team members.
    """
    team_members = crud.team_member.get_active(db, skip=skip, limit=limit)
    return team_members


@router.post("/", response_model=schemas.TeamMember)
def create_team_member(
    *,
    db: Session = Depends(deps.get_db),
    team_member_in: schemas.TeamMemberCreate,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Create new team member.
    """
    team_member = crud.team_member.create(db, obj_in=team_member_in)
    return team_member


@router.get("/{id}", response_model=schemas.TeamMember)
def read_team_member(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
) -> Any:
    """
    Get team member by ID.
    """
    team_member = crud.team_member.get(db, id=id)
    if not team_member:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Team member not found",
        )
    return team_member


@router.put("/{id}", response_model=schemas.TeamMember)
def update_team_member(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    team_member_in: schemas.TeamMemberUpdate,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Update a team member.
    """
    team_member = crud.team_member.get(db, id=id)
    if not team_member:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Team member not found",
        )
    team_member = crud.team_member.update(db, db_obj=team_member, obj_in=team_member_in)
    return team_member


@router.delete("/{id}", response_model=schemas.TeamMember)
def delete_team_member(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: models.User = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Delete a team member.
    """
    team_member = crud.team_member.get(db, id=id)
    if not team_member:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Team member not found",
        )
    team_member = crud.team_member.remove(db, id=id)
    return team_member